/*
 * MIT License
 *
 * Copyright (c) 2020 ManBang Group
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package io.manbang.wireless.thresh.channel;

import android.text.TextUtils;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 *
 * js到native bridge格式转换工具类
 *
 */
public class BridgeUtil {

    /**
     *
     * 获取bridge methodId
     *
     * @param jsParams
     * @return
     */
    public static Object getBridgeMethodId(Map jsParams){
        if (jsParams == null){
            return "-1";
        }
        // 获取一级参数
        if (jsParams.get(Methods.CALL_PARAMS) != null) {
            // 拿到 methodId 、request
            return ((HashMap) jsParams.get(Methods.CALL_PARAMS)).get("methodId");
        }
        return "-1";
    }

    /**
     * 将js bridge返回的代码转换为bridge支持的格式请求
     *
     * @param jsParams
     * @return
     *
     */
    public static String getMethod(Map jsParams){

        if (jsParams == null){
            return "";
        }
        String method = "";
        // 获取一级参数
        if (jsParams.get(Methods.CALL_PARAMS) != null){
            // 拿到 methodId 、request
            if (((HashMap )jsParams.get(Methods.CALL_PARAMS)).get("request") != null){
                // 拿到 params 、method、module

                try{
                    method = (String)((HashMap )(((HashMap )jsParams.get(
                            Methods.CALL_PARAMS)).get("request"))).get("method");
                }catch (Exception e){
                    e.printStackTrace();

                }
            }
        }

        return method;
    }
    /**
     * 将js bridge返回的代码转换为bridge支持的格式请求
     *
     * @param jsParams
     * @return
     *
     */
    public static Map jsToBridgeRequest(Map jsParams){

        if (jsParams == null){
            return null;
        }
        Map args = new HashMap();
        // 获取一级参数
        if (jsParams.get(Methods.CALL_PARAMS) != null){
            // 拿到 methodId 、request
            if (((HashMap )jsParams.get(Methods.CALL_PARAMS)).get("request") != null){
                // 拿到 params 、method、module
                args.put("module",((HashMap )(((HashMap )jsParams.get(
                        Methods.CALL_PARAMS)).get("request"))).get("module"));
                Object business = ((HashMap )(((HashMap )jsParams.get(
                        Methods.CALL_PARAMS)).get("request"))).get("business");
                if (business != null && business instanceof String && !TextUtils.isEmpty((String)business)){
                    args.put("business",(String)business);
                }
                args.put("method",((HashMap )(((HashMap )jsParams.get(
                        Methods.CALL_PARAMS)).get("request"))).get("method"));
                if (((HashMap )(((HashMap )jsParams.get(Methods.CALL_PARAMS)).get("request"))).get(
                        Methods.CALL_PARAMS) != null){
                    // 拿到 url 、method、data
                    Map params = new HashMap();
                    for (Iterator<Map.Entry<String, Object>> iterator = (((HashMap )((HashMap )(((HashMap )jsParams.get(
                            Methods.CALL_PARAMS)).get("request"))).get(Methods.CALL_PARAMS))).entrySet().iterator(); iterator.hasNext(); ) {
                        Map.Entry<String, Object> entry = iterator.next();
                        if ("data".equals(entry.getKey())){
                            params.put("data",entry.getValue());
                        }else {
                            if (entry.getValue() != null){
                                params.put(entry.getKey(), entry.getValue());
                            }
                        }
                    }
                    // bridge底层逻辑
                    if (!params.containsKey("data")){
                        params.put("data",new HashMap<>());
                    }
                    args.put("params",params);
                }
            }
            return args;
        }

        return null;
    }

    /**
     *
     *
     * 组装参数返回给js
     *
     * {
     *   method: 'bridgeResponse',
     *   params: {
     *     methodId: 1,
     *     response: {
     *       code: 200,
     *       reason: '',
     *       data: {}
     *     }
     *   }
     * }
     * @return
     */
    public static Map bridgeResponseToJS(Object methodId,Map bridgeResponse){

        if (bridgeResponse == null){
            return null;
        }
        Map args = new HashMap();
        Map params = new HashMap();
        params.put("methodId",methodId);
        params.put("response",bridgeResponse);
        args.put("method", Methods.BRIDGE_RESPONSE);
        args.put("params",params);
        return args;
    }

}
