/*
 * MIT License
 *
 * Copyright (c) 2020 ManBang Group
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package io.manbang.wireless.thresh.channel;

/**
 *      js端通信方法名：MethodChannel_Js
 *      flutter端通信方法名：MethodChannel_Flutter
 *      参数是一个具有 method 和 args 属性的 Map，method是字符串，args是任意类型
 *      如果method=='reload'，则重新加载bundle.js
 *      如果method=='print'，则在native输出args的值，并把Map完整传输到另一端的MethodChannel中
 *      如果method为其他值，则只把Map完整传输到另一端
 *
 */
public class Methods {

    /**
     * flutter调用js
     */
    public static final String METHOD_CHANNEL_FLUTTER_CALL_JS  = "methodChannel_flutter_call_js";
    /**
     * flutter调用native
     */
    public static final String METHOD_CHANNEL_FLUTTER_CALL_NATIVE = "methodChannel_flutter_call_native";
    /**
     * native 注入到当前 js context 中的方法
     */
    public static final String METHOD_CHANNEL_JS_CALL_FLUTTER = "methodChannel_js_call_flutter";
    /**
     * 通知js刷新页面
     */
    public static final String METHOD_CHANNEL_FIRE_JS_EVENT = "methodChannel_fire_js_event";
    /**
     * native 注入到当前 js context 中的方法
     */
    public static final String METHOD_CHANNEL_JS_CALL_NATIVE  = "methodChannel_js_call_native";
    /**
     * 用来做 bridge回调
     */
    public static final String METHOD_CHANNEL_NATIVE_CALL_JS  = "methodChannel_native_call_js";

    /**
     * native调js方法
     */
    public static final String RELOAD = "reload";
    /**
     * native调js方法
     */
    public static final String PRINT = "print";

    /**
     * native调js方法
     * jsCallNative 新增触发 nativeView 方法，参数格式如下：
     *
     * {
     *   method: 'invokeNativeViewMethod', // 固定
     *   params: {
     *     methodName: 'refresh', // 需要触发的 native view 方法名，目前值有 refresh destroy，必传
     *     methodParams: {}, // 触发 native view 方法时传递的参数，非必传
     *     viewType: 'trade/short_distance_map_view', // 触发的 native view type，必传
     *     viewParams: {}, // 触发的 native view 当前的参数，存在参数时会传
     *   }
     */
    public static final String REFRESH_NATIVE_VIEW = "invokeNativeViewMethod";

    /**
     * 页面渲染完成
     */
    public static final String PAGE_DID_SHOW = "pageDidShow";

    public static final String CALL_PARAMS = "params";

    public static final String CALL_METHOD = "method";

    public static final String BRIDGE_REQUEST = "bridgeRequest";

    public static final String BRIDGE_RESPONSE = "bridgeResponse";

    public static final String callNative = "callNative";

}

