/*
 * MIT License
 *
 * Copyright (c) 2020 ManBang Group
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package io.manbang.wireless.thresh.channel;

import android.content.Context;

import io.manbang.wireless.thresh.jsruntime.JSEngine;
import io.manbang.wireless.thresh.jsruntime.V8Util;
import io.manbang.wireless.thresh.load.BundleCallback;
import io.manbang.wireless.thresh.load.BundleLoader;
import io.manbang.wireless.thresh.util.ThreshLogger;
import io.manbang.wireless.thresh.util.ThreshToast;

import java.util.Map;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.PluginRegistry;
import io.flutter.view.FlutterView;

/**
 * 动态flutter methodChannel通道
 */
public class ThreshMethodChannel {

    public MethodChannel mDynamicChannel;

    private Context mContext;

    private JSEngine dynamicJSEngine;

    private ThreshBridgeApapter dynamicFBridgeAdapter;

    /**
     * 构造函数 flutterview模式
     */
    public ThreshMethodChannel(Context context, FlutterView flutterView, JSEngine dynamicJSEngine,
                               ThreshBridgeApapter dynamicFBridgeAdapter) {
        mContext = context;
        mDynamicChannel = new MethodChannel(flutterView, ChannelType.DYNAMIC_CHANNEL.getType());
        this.dynamicJSEngine = dynamicJSEngine;
        this.dynamicFBridgeAdapter = dynamicFBridgeAdapter;
    }

    /**
     * 构造函数 activity插件模式
     *
     * @param context
     * @param registrar
     * @param dynamicJSEngine
     */
    public ThreshMethodChannel(Context context, PluginRegistry.Registrar registrar,
                               JSEngine dynamicJSEngine) {
        mContext = context;
        mDynamicChannel = new MethodChannel(registrar.messenger(),
                                            ChannelType.DYNAMIC_CHANNEL.getType());
        this.dynamicJSEngine = dynamicJSEngine;
    }

    /**
     * 通道注册
     */
    public void register(JSEngine dynamicJSEngine) {
        dynamicJSEngine.setMethodCall(mDynamicChannel);
        mDynamicChannel.setMethodCallHandler(new MethodChannel.MethodCallHandler() {
            @Override
            public void onMethodCall(MethodCall methodCall, MethodChannel.Result result) {
                procMethodCall(methodCall, result);
            }
        });
        ThreshLogger.i("thresh_channel_register");
    }

    /**
     * 通道解绑
     */
    public void unRegister() {
        mDynamicChannel.setMethodCallHandler(null);
        mDynamicChannel = null;
    }
    private void procMethodCall(MethodCall methodCall, MethodChannel.Result result){

        ThreshLogger.i("thresh_channel_method_call method= " + methodCall.method  + "methodCall.arguments = "+ methodCall.arguments);
        if (methodCall == null){
            return;
        }
        if (Methods.METHOD_CHANNEL_FLUTTER_CALL_JS.equals(methodCall.method)) {
            dynamicJSEngine.getJsExecutor().executeJSFunction(Methods.METHOD_CHANNEL_FLUTTER_CALL_JS, null,
                                                              V8Util.toV8Object(dynamicJSEngine.getJsExecutor().ensureInitV8(), (Map) methodCall.arguments));
        } else if (Methods.METHOD_CHANNEL_FLUTTER_CALL_NATIVE.equals(methodCall.method)
                && methodCall.arguments != null && methodCall.arguments instanceof Map) {
            if (Methods.RELOAD.equals(((Map) methodCall.arguments).get(Methods.CALL_METHOD))) {
                // 测试模式
                if (dynamicJSEngine.getJsExecutor().isDebug()){
                    BundleLoader.loadBundleJsDebug(dynamicJSEngine, new BundleCallback() {
                        @Override
                        public void onResult(int code, String reason) {
                            if (code != 0) {
                                ThreshToast.makeText(mContext, reason, 0);
                            }
                        }
                    });
                } else {
                    dynamicJSEngine.getJsExecutor().executeScriptFile(dynamicJSEngine.bundlePath + "/bundle.js", null);
                }
            }else if (Methods.callNative.equals(((Map) methodCall.arguments).get(Methods.CALL_METHOD))){
                if (dynamicFBridgeAdapter != null){
                    // 处理自定义bridge通道
                }
            } else if (Methods.PRINT.equals(((Map) methodCall.arguments).get(Methods.CALL_METHOD))) {
                ThreshLogger.d("methodCall.method = " + methodCall.method, result.toString());
            }
        } else {
            ThreshLogger.e("Methods.METHOD_CHANNEL_NATIVE_CALL_JS", "procMethodCall null" + methodCall.method);
        }
    }

}