/*
 * MIT License
 *
 * Copyright (c) 2020 ManBang Group
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

package io.manbang.wireless.thresh.jsruntime;

import android.content.Context;
import android.os.Handler;
import android.os.Looper;
import android.text.TextUtils;

import com.eclipsesource.v8.JavaCallback;
import com.eclipsesource.v8.V8Array;
import com.eclipsesource.v8.V8Object;
import io.manbang.wireless.thresh.ThreshActivity;
import io.manbang.wireless.thresh.channel.BridgeUtil;
import io.manbang.wireless.thresh.channel.ThreshBridgeApapter;
import io.manbang.wireless.thresh.channel.Methods;
import io.manbang.wireless.thresh.util.ThreshLogger;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import io.flutter.plugin.common.MethodChannel;

/**
 * 动态js 渲染引擎
 */
public class JSEngine {

    private static final String TAG = "JSEngine";
    public String bundlePath;
    public long mStartTime;
    public String mBizName;
    public Context mContext;
    public ThreshBridgeApapter dynamicFBridgeAdapter;
    public MethodChannel dynamicChannel;
    public Handler mMainHandler;
    public List<String> mEventType;
    private JSExecutor jsExecutor;

    public JSEngine(String bizName, Context context,
                    ThreshBridgeApapter dynamicFBridgeAdapter) {
        mStartTime = System.currentTimeMillis();
        mContext = context;
        mBizName = bizName;
        this.dynamicFBridgeAdapter = dynamicFBridgeAdapter;
        jsExecutor = new JSExecutor();
        initJSEngine();
        mMainHandler = new Handler(Looper.getMainLooper());
        mEventType = new ArrayList<>();
        // 当前框架中支持的事件名
        mEventType.add("pageOnShow");
        mEventType.add("pageOnHide");
        // 安卓使用
        mEventType.add("nativePop");
        // iOS使用
        mEventType.add("willDestroy");
    }

    public static void monitorPageRender(Map<String, Object> objectMap, long startTime) {
        if (objectMap == null) {
            return;
        }

        String pageName = objectMap.containsKey("pageName") ? (String) objectMap.get(
                "pageName") : "";

        long pageDidLoad;
        if (objectMap.get("pageDidLoad") instanceof Integer) {
            pageDidLoad = objectMap.containsKey("pageDidLoad") ? ((Integer) objectMap.get(
                    "pageDidLoad")).intValue() : 0;
        } else if (objectMap.get("pageDidLoad") instanceof Long) {
            pageDidLoad = objectMap.containsKey("pageDidLoad") ? ((Long) objectMap.get(
                    "pageDidLoad")).longValue() : 0;
        } else {
            pageDidLoad = 0L;
        }

        long networkTime;
        if (objectMap.get("networkTime") instanceof Integer) {
            networkTime = objectMap.containsKey("networkTime") ? ((Integer) objectMap.get(
                    "networkTime")).intValue() : 0;
        } else if (objectMap.get("networkTime") instanceof Long) {
            networkTime = objectMap.containsKey("networkTime") ? ((Long) objectMap.get(
                    "networkTime")).longValue() : 0;
        } else {
            networkTime = 0L;
        }
        long pageDidShowTime;
        if (objectMap.get("pageDidShowTime") instanceof Integer) {
            pageDidShowTime = objectMap.containsKey("pageDidShowTime") ? ((Integer) objectMap.get(
                    "pageDidShowTime")).intValue() : 0;
        } else if (objectMap.get("pageDidShowTime") instanceof Long) {
            pageDidShowTime = objectMap.containsKey("pageDidShowTime") ? ((Long) objectMap.get(
                    "pageDidShowTime")).longValue() : 0;
        } else {
            pageDidShowTime = 0L;
        }

        long jsStartTime;
        if (objectMap.get("jsStartTime") instanceof Integer) {
            jsStartTime = objectMap.containsKey("jsStartTime") ? ((Integer) objectMap.get(
                    "jsStartTime")).intValue() : 0;
        } else if (objectMap.get("jsStartTime") instanceof Long) {
            jsStartTime = objectMap.containsKey("jsStartTime") ? ((Long) objectMap.get(
                    "jsStartTime")).longValue() : 0;
        } else {
            jsStartTime = 0L;
        }

        long jsRunAppTime;
        if (objectMap.get("jsRunAppTime") instanceof Integer) {
            jsRunAppTime = objectMap.containsKey("jsRunAppTime") ? ((Integer) objectMap.get(
                    "jsRunAppTime")).intValue() : 0;
        } else if (objectMap.get("jsRunAppTime") instanceof Long) {
            jsRunAppTime = objectMap.containsKey("jsRunAppTime") ? ((Long) objectMap.get(
                    "jsRunAppTime")).longValue() : 0;
        } else {
            jsRunAppTime = 0L;
        }

        long channelFirstSpendTime;
        if (objectMap.get("channelFirstSpendTime") instanceof Integer) {
            channelFirstSpendTime = objectMap.containsKey(
                    "channelFirstSpendTime") ? ((Integer) objectMap.get("channelFirstSpendTime"))
                    .intValue() : 0;
        } else if (objectMap.get("channelFirstSpendTime") instanceof Long) {
            channelFirstSpendTime = objectMap.containsKey(
                    "channelFirstSpendTime") ? ((Long) objectMap.get("channelFirstSpendTime"))
                    .longValue() : 0;
        } else {
            channelFirstSpendTime = 0L;
        }

        String flutterVersion = objectMap.containsKey("flutterVersion") ? (String) objectMap.get(
                "flutterVersion") : "";

        String jsVersion = objectMap.containsKey("jsVersion") ? (String) objectMap.get(
                "jsVersion") : "";
        long currentTime = System.currentTimeMillis();
        long rendTime = pageDidLoad - startTime;
        ThreshLogger.i("thresh init_flutter",
                       "flutter_init :" + " currentRealTime =" + (currentTime - startTime) + " startTime =" + startTime + " all_load_time =" + (rendTime + networkTime) + " render_time =" + rendTime + " network_time =" + networkTime + " jsStartTime =" + (pageDidShowTime - jsStartTime) + " jsRunAppTime =" + (pageDidShowTime - jsRunAppTime) + " channelFirstSpendTime =" + channelFirstSpendTime);
    }

    public void setMethodCall(MethodChannel dynamicChannel) {
        this.dynamicChannel = dynamicChannel;
    }

    private void initJSEngine() {

        jsExecutor.registerObj();

        jsExecutor.registerJavaMethod(new JavaCallback() {
            @Override
            public Object invoke(V8Object receiver, V8Array parameters) {
                jsCallFlutter(parameters);
                return null;
            }
        }, Methods.METHOD_CHANNEL_JS_CALL_FLUTTER);

        jsExecutor.registerJavaMethod(new JavaCallback() {
            @Override
            public Object invoke(V8Object receiver, V8Array parameters) {
                jsCallNative(parameters);
                return null;
            }
        }, Methods.METHOD_CHANNEL_JS_CALL_NATIVE);
    }

    /**
     * js调flutter方法，native端只做中转
     */
    private synchronized void jsCallFlutter(V8Array parameters) {
        final V8Array local_args = parameters.twin();
        List<Object> list = V8Util.toList(local_args);
        ThreshLogger.d("jsCallFlutter +++++ start " + list);
        if (list != null && !list.isEmpty()) {
            Map args = new HashMap();
            for (Iterator<Map.Entry<String, Object>> iterator = ((HashMap) (list.get(0))).entrySet()
                    .iterator(); iterator.hasNext(); ) {
                Map.Entry<String, Object> entry = iterator.next();
                args.put(entry.getKey(), entry.getValue());
            }
            mMainHandler.post(new Runnable() {
                @Override
                public void run() {
                    if (dynamicChannel != null) {
                        dynamicChannel.invokeMethod(Methods.METHOD_CHANNEL_JS_CALL_FLUTTER, args);
                    }
                }
            });
            ThreshLogger.d("js_engine_js_call_flutter_not_null" + args);
        } else {
            ThreshLogger.d("js_engine_js_call_flutter_null");
        }


    }

    /**
     * js调native方法
     * <p>
     * 目前用于：
     * <p>
     * print ：打印
     * <p>
     * reload ：重新加载js
     * <p>
     * bridgeRequest : bridge通道
     */
    private synchronized void jsCallNative(V8Array parameters) {
        final V8Array local_args = parameters.twin();
        List<Object> list = V8Util.toList(local_args);
        ThreshLogger.d("jsCallNative +++++ start " + list);
        if (list != null && !list.isEmpty()) {
            Map args = new HashMap();
            for (Iterator<Map.Entry<String, Object>> iterator = ((HashMap) (list.get(0))).entrySet()
                    .iterator(); iterator.hasNext(); ) {
                Map.Entry<String, Object> entry = iterator.next();
                args.put(entry.getKey(), entry.getValue());
            }
            ThreshLogger.d("jsCallNative args = " + args);
            if (Methods.BRIDGE_REQUEST.equals(args.get(Methods.CALL_METHOD))) {
                if (dynamicFBridgeAdapter != null) {
                    // 自定义bridge拦截
                }
            } else if (Methods.RELOAD.equals(args.get(Methods.CALL_METHOD))) {
                ThreshLogger.d("methodCall.method = " + args);
            } else if (Methods.PRINT.equals(args.get(Methods.CALL_METHOD))) {
                ThreshLogger.d("methodCall.method = " + args);
            } else if (Methods.PAGE_DID_SHOW.equals(args.get(Methods.CALL_METHOD))) {
                if (mStartTime > 0L) {
                    if (args.get(Methods.CALL_PARAMS) instanceof HashMap) {
                        monitorPageRender((HashMap) args.get(Methods.CALL_PARAMS), mStartTime);
                    }
                    mStartTime = 0L;
                }
                ThreshLogger.d("methodCall.method = " + args);
            } else if (Methods.REFRESH_NATIVE_VIEW.equals(args.get(Methods.CALL_METHOD))) {
                ThreshLogger.d("methodCall.method = " + args);
                if (mContext == null) {
                    return;
                }
                ((ThreshActivity) mContext).updateNativeView(
                        (HashMap) args.get(Methods.CALL_PARAMS));
            } else {
                ThreshLogger.d("methodCall.method = " + args);
            }

        } else {
            ThreshLogger.d("js_engine_js_call_native_null");
        }
    }

    private void execJS(Map args, Map<String, Object> bridgeResult) {
        if (getJsExecutor() == null) {
            return;
        }
        if (mContext == null) {
            return;
        }
        mMainHandler.post(new Runnable() {
            @Override
            public void run() {
                if (getJsExecutor() == null) {
                    return;
                }
                getJsExecutor().executeJSFunction(Methods.METHOD_CHANNEL_NATIVE_CALL_JS, null,
                                                  V8Util.toV8Object(getJsExecutor().ensureInitV8(),
                                                                    BridgeUtil.bridgeResponseToJS(
                                                                            BridgeUtil
                                                                                    .getBridgeMethodId(
                                                                                            args),
                                                                            bridgeResult)));
            }
        });
    }

    /**
     * 刷新指定页面
     */
    public void pageOnShow() {
        if (jsExecutor == null) {
            return;
        }
        jsExecutor.executeJSFunction(Methods.METHOD_CHANNEL_FIRE_JS_EVENT, null, "pageOnShow");
    }

    /**
     * 返回上一级页面
     */
    public void nativePop() {
        if (jsExecutor == null) {
            return;
        }
        jsExecutor.executeJSFunction(Methods.METHOD_CHANNEL_FIRE_JS_EVENT, null, "nativePop");
    }

    /**
     *
     */
    public void pageOnHide() {
        if (jsExecutor == null) {
            return;
        }
        jsExecutor.executeJSFunction(Methods.METHOD_CHANNEL_FIRE_JS_EVENT, null, "pageOnHide");
    }

    /**
     * 执行指定方法
     */
    public void execJS(String method, Object params) {
        if (jsExecutor == null) {
            return;
        }
        ThreshLogger.i(TAG, "execJS = method start ");
        ThreshLogger.i(TAG, "execJS = method : " + method + " , params :" + params);
        if (!TextUtils.isEmpty(method) && mEventType != null && mEventType.contains(method)) {
            jsExecutor.executeJSFunction(Methods.METHOD_CHANNEL_FIRE_JS_EVENT, null, method);
        } else {
            Map destroyMaps = new HashMap();
            destroyMaps.put("method", method);
            destroyMaps.put("params", params);
            jsExecutor.executeJSFunction(Methods.METHOD_CHANNEL_NATIVE_CALL_JS, null,
                                         V8Util.toV8Object(jsExecutor.ensureInitV8(), destroyMaps));
        }
        ThreshLogger.i(TAG, "execJS = method end ");
    }

    /**
     * 销毁关闭页面的执行方法
     */
    public void onDestory() {
        if (jsExecutor == null) {
            return;
        }
        Map destroyMaps = new HashMap();
        destroyMaps.put("method", "onDestroyed");
        destroyMaps.put("params", null);
        jsExecutor.executeJSFunction(Methods.METHOD_CHANNEL_NATIVE_CALL_JS, null,
                                     V8Util.toV8Object(jsExecutor.ensureInitV8(), destroyMaps));
    }

    public void close() {
        if (jsExecutor != null) {
            jsExecutor.close();
            jsExecutor = null;
        }
    }

    public JSExecutor getJsExecutor() {
        return jsExecutor;
    }

}