/*
 * MIT License
 *
 * Copyright (c) 2020 ManBang Group
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package io.manbang.wireless.thresh.util;

import android.util.Log;

/**
 * 日志
 */
public class ThreshLogger {

    /** 默认日志TAG. * */
    private final static String TAG = "ThreshLogger";
    /**
     * @Description:
     *
     */
    public static void v(String msg) {
        if (msg != null) {
            Log.v(TAG, msg);
        }
    }

    /**
     * @Description:
     *
     */
    public static void v(Object object, String msg) {
        if (msg != null) {
            Log.v(getTag(object), msg);
        }
    }

    /**
     * @Description:
     *
     */
    public static void d(String msg) {
        if (msg != null) {
            Log.d(TAG, msg);
        }
    }

    /**
     * @Description:
     *
     */
    public static void d(Object object, String msg) {
        if (msg != null) {
            Log.d(getTag(object), msg);
        }
    }

    /**
     * @Description:
     *
     */
    public static void i(String msg) {
        if (msg != null) {
            Log.i(TAG, msg);
        }
    }

    /**
     * @Description:
     *
     */
    public static void i(Object object, String msg) {
        if (msg != null) {
            Log.i(getTag(object), msg);
        }
    }

    /**
     * @Description:
     *
     */
    public static void w(String msg) {
        if (msg != null) {
            Log.w(TAG, msg);
        }
    }

    /**
     * @Description:
     *
     */
    public static void w(Object object, String msg) {
        if (msg != null) {
            Log.w(getTag(object), msg);
        }
    }

    /**
     * @Description:
     */
    public static void w(Object object, Throwable error) {
        if (error != null) {
            Log.w(getTag(object), error.getMessage(), error);
        }
    }

    /**
     * @Description:
     */
    public static void e(String msg) {
        if (msg != null) {
            Log.w(TAG, msg);
        }
    }

    /**
     * @Description:
     */
    public static void e(Object object, String msg) {
        if (msg != null) {
            Log.e(getTag(object), msg);
        }
    }

    /**
     * @Description:
     *
     */
    public static void e(Object object, Throwable error) {
        if (error != null) {
            Log.e(getTag(object), error.getMessage(), error);
        }
    }

    /**
     * @return 可定位的异常 @Description:
     *     <p>private static Throwable filterThrowable(Throwable error) { StackTraceElement[] ste =
     *     error.getStackTrace(); error.setStackTrace(new StackTraceElement[] { ste[0] }); return
     *     error; }
     */

    /** @return Log TAG. @Description: */
    private static String getTag(Object object) {
        if (object == null) {
            return TAG;
        }
        String name = object.getClass().getSimpleName();
        if ("String".equals(name)) {
            return object.toString();
        }
        return name;
    }
}
